package com.kakao.sdk.auth

import com.google.gson.JsonObject
import com.kakao.sdk.auth.model.OAuthToken
import com.kakao.sdk.auth.model.AccessTokenResponse
import com.kakao.sdk.common.KakaoJson
import com.kakao.sdk.common.KakaoSdk
import com.kakao.sdk.common.util.AESCipher
import com.kakao.sdk.common.util.Cipher
import com.kakao.sdk.common.util.PersistentKVStore
import com.kakao.sdk.common.util.SharedPrefsWrapper
import java.util.*

/**
 * @suppress
 * @author kevin.kang. Created on 2018. 3. 27..
 */
class DefaultAccessTokenRepo(
    val appCache: PersistentKVStore = SharedPrefsWrapper(
        KakaoSdk.applicationContextInfo.sharedPreferences
    ),
    val encryptor: Cipher = AESCipher()
) : AccessTokenRepo {
    init {
        val version = appCache.getString(versionKey)
        if (version == null) {
            migrateFromOldVersion()
        }
    }

    override fun clearCache() {
        appCache.remove(atKey).remove(rtKey).remove(
            atExpiresAtKey
        ).remove(rtExpiresAtKey)
            .remove(scopesKey).commit()
    }

    override fun toCache(accessTokenResponse: AccessTokenResponse): OAuthToken {
        accessTokenResponse.refreshToken?.let { appCache.putString(rtKey, encryptor.encrypt(it)) }
        accessTokenResponse.refreshTokenExpiresIn?.let {
            appCache.putLong(
                rtExpiresAtKey,
                Date().time + 1000L * it
            )
        }
        accessTokenResponse.scopes?.let { appCache.putString(scopesKey, it) }

        appCache.putString(atKey, encryptor.encrypt(accessTokenResponse.accessToken))
            .putLong(atExpiresAtKey, Date().time + 1000L * accessTokenResponse.accessTokenExpiresIn)
            .putString(versionKey, "2.0.0").commit()
        val token = fromCache()
        return token
    }

    override fun fromCache(): OAuthToken {
        val at = appCache.getString(atKey, null)?.let { encryptor.decrypt(it) }
        val rt = appCache.getString(rtKey, null)?.let { encryptor.decrypt(it) }
        val atExpiresAt = Date(appCache.getLong(atExpiresAtKey, 0))
        val rtExpiresAt = Date(appCache.getLong(rtExpiresAtKey, Long.MAX_VALUE))
        val scopes = appCache.getString(scopesKey, null)?.split(' ')
        return OAuthToken(at, atExpiresAt, rt, rtExpiresAt, scopes)
    }

    companion object {
        const val atKey = "com.kakao.token.OAuthToken"
        const val rtKey = "com.kakao.token.RefreshToken"
        const val atExpiresAtKey = "com.kakao.token.OAuthToken.ExpiresAt"
        const val rtExpiresAtKey = "com.kakao.token.RefreshToken.ExpiresAt"
        const val secureModeKey = "com.kakao.token.KakaoSecureMode"
        const val scopesKey = "com.kakao.token.Scopes"

        const val versionKey = "com.kakao.token.Version"
    }

    private fun migrateFromOldVersion() {
        val secureMode = appCache.getString(secureModeKey, "false")
        val at = appCache.getString(atKey, null)?.let {
            val legacyAt =
                KakaoJson.fromJson<JsonObject>(it, JsonObject::class.java)["value"].asString
            if (secureMode == "false") encryptor.encrypt(legacyAt) else legacyAt
        }
        val rt = appCache.getString(rtKey, null)?.let {
            val legacyRt =
                KakaoJson.fromJson<JsonObject>(it, JsonObject::class.java)["value"].asString
            if (secureMode == "false") encryptor.encrypt(legacyRt) else legacyRt
        }
        val atExpiresAt = appCache.getString(atExpiresAtKey, null)?.let {
            KakaoJson.fromJson<JsonObject>(it, JsonObject::class.java)["value"].asLong
        } ?: 0
        val rtExpiresAt = appCache.getString(rtExpiresAtKey, null)?.let {
            KakaoJson.fromJson<JsonObject>(it, JsonObject::class.java)["value"].asLong
        } ?: 0

        at?.let { appCache.putString(atKey, it) }
        rt?.let { appCache.putString(rtKey, it) }
        appCache.putLong(atExpiresAtKey, atExpiresAt).putLong(
            rtExpiresAtKey, rtExpiresAt
        )
        appCache.remove(secureModeKey).putString(
            versionKey, "2.0.0"
        ).commit()
    }
}